package com.gic.plug.web.controller;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.constants.Constants;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.plug.web.strategy.ScreenBackStrategy;
import com.gic.plug.web.strategy.impl.*;
import com.gic.search.business.api.constant.enums.OperateEnum;
import com.gic.store.constant.StoreESFieldsEnum;
import com.gic.widget.screening.api.dto.*;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.gic.widget.screening.api.service.EsScreeningInitService;
import com.gic.widget.screening.api.service.EsScreeningSearchService;
import com.gic.widget.screening.api.service.EsScreeningTemplateService;

import javax.annotation.PostConstruct;

/**
 *
 */
@Controller
public class ScreeningController {

    private Logger LOGGER = LogManager.getLogger(ScreeningController.class);

    @Autowired
    private EsScreeningInitService esScreeningInitService;
    @Autowired
    private EsScreeningTemplateService esScreeningTemplateService;
    @Autowired
    private StoreAreaBackImpl storeAreaBack;
    @Autowired
    private StoreBackImpl storeBack;
    @Autowired
    private StoreBrandBackImpl storeBrandBack;
    @Autowired
    private StoreCustomBackImpl storeCustomBack;
    @Autowired
    private StoreGroupBackImpl storeGroupBack;
    @Autowired
    private StoreRegionBackImpl storeRegionBack;
    @Autowired
    private StoreSelectBackImpl storeSelectBack;
    @Autowired
    private StoreTagBackImpl storeTagBack;
    private Map<String, ScreenBackStrategy> strategyMap = new HashMap<>();

    @PostConstruct
    public void init(){
        strategyMap.put(StoreESFieldsEnum.AREAIDS.getField(), storeAreaBack);
        strategyMap.put(StoreESFieldsEnum.STOREID.getField(), storeBack);
        strategyMap.put(StoreESFieldsEnum.STOREBRANDIDLIST.getField(), storeBrandBack);
        strategyMap.put(StoreESFieldsEnum.STOREGROUPIDLIST.getField(), storeGroupBack);
        strategyMap.put(StoreESFieldsEnum.REGIONID.getField(), storeRegionBack);
        strategyMap.put(StoreESFieldsEnum.STATUS.getField(), storeSelectBack);
        strategyMap.put(StoreESFieldsEnum.COMPLETESTATUS.getField(), storeSelectBack);
        strategyMap.put(StoreESFieldsEnum.CREATETYPE.getField(), storeSelectBack);
        strategyMap.put(StoreESFieldsEnum.STORETYPE.getField(), storeSelectBack);
        strategyMap.put(StoreESFieldsEnum.ERPSTATUS.getField(), storeSelectBack);
        strategyMap.put(StoreESFieldsEnum.STORETAG.getField(), storeTagBack);
        strategyMap.put(StoreESFieldsEnum.C1.getField(), storeCustomBack);
        strategyMap.put(StoreESFieldsEnum.C2.getField(), storeCustomBack);
        strategyMap.put(StoreESFieldsEnum.C3.getField(), storeCustomBack);
        strategyMap.put(StoreESFieldsEnum.C4.getField(), storeCustomBack);
        strategyMap.put(StoreESFieldsEnum.C5.getField(), storeCustomBack);
        strategyMap.put(StoreESFieldsEnum.C6.getField(), storeCustomBack);
        strategyMap.put(StoreESFieldsEnum.C7.getField(), storeCustomBack);
        strategyMap.put(StoreESFieldsEnum.C8.getField(), storeCustomBack);
        strategyMap.put(StoreESFieldsEnum.C9.getField(), storeCustomBack);
        strategyMap.put(StoreESFieldsEnum.C10.getField(), storeCustomBack);
        System.out.println("对象初始化完成");
    }
    /**
     * @return
     * @throws
     * @Title: getScreeningTemplate
     * @Description: 获取模板
     * @author xugh
     */
    @RequestMapping("get-screening-template")
    @ResponseBody
    public RestResponse getScreeningTemplate() {
        List<EsScreeningTemplateDTO> list = this.esScreeningTemplateService.queryEsScreeningTemplateList();
        return RestResponse.success(list);
    }

    /**
     * @param sceneCode
     * @return
     * @throws
     * @Title: getScreeningInitData
     * @Description: 获取初始化筛选条件
     * @author xugh
     */
    @RequestMapping("get-screening-init-data")
    @ResponseBody
    public RestResponse getScreeningInitData(@RequestParam(defaultValue = "member") String sceneCode) {
        String enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId().toString();
        if (Constants.STORE_SCENECODE.equals(sceneCode)) {
            enterpriseId = Constants.INDEX_ENTERPRISEID;
        }
        List<EsScreeningEnterpriseCategorySceneInfoDTO> list = this.esScreeningInitService
                .queryEsScreeningByEnterpriseAndScene(enterpriseId, sceneCode);
        List<EsScreeningChainDetailDTO> all = new ArrayList<>();
        for (EsScreeningEnterpriseCategorySceneInfoDTO dto : list) {
            String categoryId = dto.getSceneCategoryId();
            EsScreeningChainDetailDTO item = this.esScreeningInitService.findRootChainByCategoryId(categoryId);
            all.add(item);
        }
        return RestResponse.success(all);
    }

    @RequestMapping("get-screening-data")
    @ResponseBody
    public RestResponse getScreeningData(String widgetChainId) {
        List<EsScreeningChainDetailDTO> list = this.esScreeningInitService
                .queryEsScreeningByParentChainId(widgetChainId);
        return RestResponse.success(list);
    }

    @RequestMapping("get-screening-chain-detail")
    @ResponseBody
    public RestResponse getScreeningChainDetail(String parentChainId, String widgetFieldKey) {
        EsScreeningChainDetailDTO dto = this.esScreeningInitService.findChainDetail(parentChainId, widgetFieldKey);
        return RestResponse.success(dto);
    }


    @RequestMapping("get-screening-widget-chain-detail")
    @ResponseBody
    public Object getScreeningWidgetDetail(String widgetChainId) {
        EsScreeningChainDetailDTO dto = this.esScreeningInitService.findChainDetail(widgetChainId);
        return RestResponse.success(dto);
    }

    @RequestMapping("screen-back")
    @ResponseBody
    public String screenBack(String param) throws ClassNotFoundException {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        JSONArray array = JSON.parseArray(param);
        StringBuilder backMessage = new StringBuilder();
        for(int i=0; i< array.size(); i++){
            JSONObject jsonObject = array.getJSONObject(i);
            String type = jsonObject.getString("type");
            if(StringUtils.isNotBlank(type)){
                if(OperateEnum.OPERATE_AND_NOT.getValue().equals(type)){
                    backMessage.append(OperateEnum.OPERATE_AND_NOT.getDescribe()).append(" ");
                }
                if(OperateEnum.OPERATE_OR.getValue().equals(type)){
                    backMessage.append(OperateEnum.OPERATE_OR.getDescribe()).append(" ");
                }
            }
            JSONArray list = jsonObject.getJSONArray("list");
            StringBuilder stringBuilder = new StringBuilder("(");
            for(int j = 0; j<list.size(); j++){
                JSONObject json = list.getJSONObject(j);
                String key = json.getJSONObject("data").getString("key");
                String value = json.getJSONObject("data").getString("value");
                String compute = json.getJSONObject("data").getString("compute");
                String message = strategyMap.get(key).screenBack(compute, value, key, enterpriseId);
                stringBuilder.append(message);
                if(j < list.size()-1){
                    stringBuilder.append("并且");
                }
            }
            stringBuilder.append(")");
            backMessage.append(stringBuilder);
        }
        return backMessage.toString();
    }
}
